<div class="input-group">
    {% if prefix %}<div class="input-group-text">{{ prefix }}</div>{% endif %}
    {% include widget.subwidgets.0.template_name with widget=widget.subwidgets.0 %}
    <div class="input-group-text">{{ postfix }} – {{ prefix }}</div>
    {% include widget.subwidgets.1.template_name with widget=widget.subwidgets.1 %}
    {% if postfix %}<div class="input-group-text">{{ postfix }}</div>{% endif %}
</div>
