{% load static %}
<!DOCTYPE html>
<html lang="{{ request.LANGUAGE_CODE }}">
    <head>
        <title>
            {% block title %}
                Data Show
            {% endblock title %}
        </title>
        {% block css %}
            <link href="https://cdn.jsdelivr.net/npm/bootstrap@5.3.3/dist/css/bootstrap.min.css"
                  rel="stylesheet"
                  integrity="sha384-QWTKZyjpPEjISv5WaRU9OFeRpok6YctnYmDr5pNlyT2bRjXh0JMhjY6hW+ALEwIH"
                  crossorigin="anonymous">
            <link rel="stylesheet" href="{% static 'datashow/css/datashow.css' %}">
        {% endblock css %}
    </head>
    <body>
        {% block container %}
            <div class="container">
                {% block content %}
                {% endblock content %}
            </div>
        {% endblock container %}
        {% block scripts %}
            <script type="text/javascript" src="{% static 'datashow/js/htmlx.min.js' %}"></script>
        {% endblock scripts %}
    </body>
</html>
