# Generated by Django 4.2.16 on 2025-02-24 10:10

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("datashow", "0002_column_facet_count"),
    ]

    operations = [
        migrations.AddField(
            model_name="table",
            name="row_description_template",
            field=models.TextField(
                blank=True,
                help_text="Python format template for describing rows / rendered as Markdown/HTML!",
                verbose_name="Row description template",
            ),
        ),
        migrations.AlterField(
            model_name="column",
            name="formatter",
            field=models.CharField(
                choices=[
                    ("plaintext", "Plaintext"),
                    ("float", "Float"),
                    ("integer", "Integer"),
                    ("date", "Date"),
                    ("datetime", "Date and time"),
                    ("boolean", "Boolean"),
                    ("link", "Link"),
                    ("summary", "Summary"),
                    ("abbreviation", "Abbreviation"),
                    ("iframe", "IFrame"),
                ],
                default="plaintext",
                help_text="Formatter for the column.",
                max_length=255,
                verbose_name="Formatter",
            ),
        ),
    ]
