# Generated by Django 4.2.16 on 2025-01-23 09:00

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('datashow', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='column',
            name='facet_count',
            field=models.PositiveSmallIntegerField(default=0, help_text='Number of facet entries, 0 if disabled.', verbose_name='Facet Count'),
        ),
    ]
